var canvas;
var stage;
var screen_width;
var screen_height;
var bmpSeq;
var numberOfImagesLoaded = 0;
var imgMonsterARun = new Image();
var imgMonsterAIdle = new Image();
var imgLevel1BG = new Image();
var state;
var character = new SpriteSheet();
var fpsCounter;
var player;
var SCALE = 30;
var world;
var b2Vec2;

var pegs = new Array(
{x: 66,y: 145},
{x: 49,y: 248},
{x: 154,y: 139},
{x: 177,y: 231},
{x: 232,y: 86},
{x: 252,y: 161},
{x: 274,y: 238},
{x: 344,y: 53},
{x: 348,y: 112},
{x: 333,y: 178},
{x: 362,y: 275},
{x: 372,y: 345},
{x: 422,y: 80},
{x: 426,y: 191},
{x: 412,y: 305},
{x: 499,y: 90},
{x: 491,y: 239},
{x: 559,y: 145},
{x: 632,y: 99},
{x: 602,y: 205},
{x: 594,y: 266},
{x: 669,y: 163},
{x: 655,y: 262},
{x: 728,y: 183},
{x: 758,y: 268}
);

var consoleLogging = true;

var stats = new Stats();
document.body.appendChild(stats.domElement);

function init(){
	if( consoleLogging == true ) console.log(":: init ::");
	canvas = document.getElementById("nutbucket");
	
	loadImages();
	initBox2D();	
	
	//Key Control Helper
    this.controlHelper = new TAEngine.ControllerHelper();

}

function loadImages () {
	imgMonsterARun.onload = handleImageLoad;
	imgMonsterARun.onerror = handleImageError;
	imgMonsterARun.src = "images/sprites/turtle.png"
	
	imgMonsterAIdle.onload = handleImageLoad;
    imgMonsterAIdle.onerror = handleImageError;
    imgMonsterAIdle.src = "images/sprites/turtle.png";
	
	imgLevel1BG.onload = handleImageLoad;
	imgLevel1BG.onerror = handleImageError;
	imgLevel1BG.src = "images/levels/level1.png";
}

function initBox2D() {
    b2Vec2 = Box2D.Common.Math.b2Vec2
        , b2BodyDef = Box2D.Dynamics.b2BodyDef
        , b2Body = Box2D.Dynamics.b2Body
        , b2FixtureDef = Box2D.Dynamics.b2FixtureDef
        , b2Fixture = Box2D.Dynamics.b2Fixture
        , b2World = Box2D.Dynamics.b2World
        , b2MassData = Box2D.Collision.Shapes.b2MassData
        , b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape
        , b2CircleShape = Box2D.Collision.Shapes.b2CircleShape
        , b2DebugDraw = Box2D.Dynamics.b2DebugDraw
          ;

    world = new b2World(
             new b2Vec2(0, 10)    //gravity
          , true                 //allow sleep
       );

   

    var fixDef = new b2FixtureDef;
    fixDef.density = 1.0;
    fixDef.friction = 0.5;
    fixDef.restitution = 0.2;

	//create ground
    var bodyDef = new b2BodyDef;
    bodyDef.type = b2Body.b2_staticBody;

    // positions the center of the object (not upper left!)
    bodyDef.position.x = canvas.width / 2 / SCALE;
    bodyDef.position.y = canvas.height / SCALE;

    fixDef.shape = new b2PolygonShape;

    // half width, half height. eg actual height here is 1 unit
    fixDef.shape.SetAsBox((canvas.width / SCALE) / 2, (10 / SCALE) / 2);
    world.CreateBody(bodyDef).CreateFixture(fixDef);

	//create pegs	
	var pegDef =  new b2BodyDef;
	pegDef.type = b2Body.b2_staticBody;
	
    for (var i = 0; i < pegs.length; ++i) {
		fixDef.shape = new b2CircleShape(
			.3 //radius
		 );
        pegDef.position.x = pegs[i].x / SCALE;
        pegDef.position.y = pegs[i].y / SCALE;
        world.CreateBody(pegDef).CreateFixture(fixDef);
    }

    //setup debug draw
    var debugDraw = new b2DebugDraw();
    debugDraw.SetSprite(document.getElementById("nutbucket-debug").getContext("2d"));
    debugDraw.SetDrawScale(SCALE);
    debugDraw.SetFillAlpha(0.5);
    debugDraw.SetLineThickness(1.0);
    debugDraw.SetFlags(b2DebugDraw.e_shapeBit | b2DebugDraw.e_jointBit);
    world.SetDebugDraw(debugDraw);

    setInterval(getNut, 3000);
	
	
};

function handleImageLoad(e) {
	if( consoleLogging == true ) console.log("image loaded");
    numberOfImagesLoaded++;

    // Don't start game until all images are loaded
    if (numberOfImagesLoaded == 3) {
        numberOfImagesLoaded = 0;
        startGame();
		ShowFPSCounter();
    }

}

function getNut(){
	var nutDef = new b2BodyDef;
	var nutFixDef = new b2FixtureDef;
    nutFixDef.density = 3;
    nutFixDef.friction = 0.5;
    nutFixDef.restitution = 0.4;
	nutDef.type = b2Body.b2_dynamicBody;
	nutFixDef.shape = new b2CircleShape(.6);
	nutDef.position.x = Math.random() * 25 + 2.5 ;
	nutDef.position.y = 50 / SCALE;
	world.CreateBody(nutDef).CreateFixture(nutFixDef);
}

function startGame(){
	if( consoleLogging == true ) console.log(":: Start Game ::");
	stage = new Stage(canvas);
	screen_width = canvas.width;
	screen_height = canvas.height;
	stage.mouseEventsEnabled = true;

	
	
	var background = new Bitmap();
	background.image = imgLevel1BG;
	stage.addChild(background);
	
	background.onPress = pressHandler;
	
	player = new Player();
	player.init();
	
	Ticker.addListener(window);
	Ticker.setFPS(60);
}

var tempString = "";

function pressHandler(e){
	tempString += "{x: " + e.stageX + ",y: " + e.stageY + "},";
	console.log( tempString );
}

function handleImageError(e) {
	if( consoleLogging == true ) console.log("Error Loading Image : " + e.target.src);
}

function tick() {
	
	
	fpsCounter.text = Math.round(Ticker.getMeasuredFPS()) + "FPS";
	player.update();
	stage.update();
	stats.update();

    //Box2D Update
	world.Step(
         1 / 60   //frame-rate
      , 10       //velocity iterations
      , 10       //position iterations
   );
	world.DrawDebugData();
	world.ClearForces();

	//requestAnimFrame(update);
	
}

function ShowFPSCounter(){
	fpsCounter = new Text("--FPS", "10px Arial", "#000");
    fpsCounter.x = 10;
    fpsCounter.y = 20;
	stage.addChild(fpsCounter);
}

init();
